@@ -263,7 +263,7 @@ function modules_create_table_for_c(kr_module_ud)
 					elseif arg ~= nil then
 						arg_conv = tostring(arg)
 					end
-					local ret_cstr = cb(ffi.C.the_worker.engine, kr_module, arg_conv)
+					local ret_cstr = cb(ffi.C.the_engine, kr_module, arg_conv)
 					if ret_cstr == nil then
 						return nil
 					end
@@ -355,7 +355,7 @@ cache.fssize = function ()
 	end
 	local size = tonumber(ffi.C.kr_fssize(path))
 	if size < 0 then
-		panic('cache.fssize(): %s', ffi.string(ffi.C.knot_strerror(size)))
+		panic('cache.fssize(): %s', kres.strerror(size))
 	else
 		return size
 	end
@@ -386,7 +386,7 @@ cache.clear = function (name, exact_name, rr_type, chunk_size, callback, prev_st
 		local apex_array = ffi.new('knot_dname_t *[1]')  -- C: dname **apex_array
 		local ret = ffi.C.kr_cache_closest_apex(cach, dname, false, apex_array)
 		if ret < 0 then
-			error(ffi.string(ffi.C.knot_strerror(ret))) end
+			error(kres.strerror(ret)) end
 		if not ffi.C.knot_dname_is_equal(apex_array[0], dname) then
 			local apex_str = kres.dname2str(apex_array[0])
 			rettable.not_apex = 'to clear proofs of non-existence call '
@@ -403,7 +403,7 @@ cache.clear = function (name, exact_name, rr_type, chunk_size, callback, prev_st
 		if chunk_size or callback
 			then error('cache.clear(): chunk_size and callback parameters not supported with rr_type') end
 		local ret = ffi.C.kr_cache_remove(cach, dname, rr_type)
-		if ret < 0 then error(ffi.string(ffi.C.knot_strerror(ret))) end
+		if ret < 0 then error(kres.strerror(ret)) end
 		return {count = 1}
 	end
 
@@ -424,7 +424,7 @@ cache.clear = function (name, exact_name, rr_type, chunk_size, callback, prev_st
 	-- Default callback function: repeat after 1ms
 	if callback == nil then callback =
 		function (cbname, cbexact_name, cbrr_type, cbchunk_size, cbself, cbprev_state, cbrettable)
-			if cbrettable.count < 0 then error(ffi.string(ffi.C.knot_strerror(cbrettable.count))) end
+			if cbrettable.count < 0 then error(kres.strerror(cbrettable.count)) end
 			if cbprev_state == nil then cbprev_state = { round = 0 } end
 			if type(cbprev_state) ~= 'table'
 				then error('cache.clear() callback: incorrect prev_state passed') end
@@ -514,7 +514,7 @@ modules.load('extended_error')
 -- Load keyfile_default
 trust_anchors.add_file('/etc/trusted-key.key', true)
 
-local function eval_cmd_compile(line, raw)
+local function eval_cmd_compile(line, mode)
 	-- Compatibility sandbox code loading
 	local function load_code(code)
 	    if getfenv then -- Lua 5.1
@@ -523,8 +523,19 @@ local function eval_cmd_compile(line, raw)
 	        return load(code, nil, 't', _ENV)
 	    end
 	end
+
+	-- See `ENGINE_EVAL_MODE_MAP(XX)` C-macro for possible values
 	local err, chunk
-	chunk, err = load_code(raw and 'return '..line or 'return table_print('..line..')')
+	if mode == "LUA_TABLE" then
+		chunk, err = load_code('return table_print(('..line..'))')
+	elseif mode == "RAW" then
+		chunk, err = load_code('return ('..line..')')
+	elseif mode == "JSON" then
+		chunk, err = load_code('return tojson(('..line..'))')
+	else
+		return nil, "invalid mode"
+	end
+
 	if err then
 		chunk, err = load_code(line)
 	end
@@ -532,8 +543,8 @@ local function eval_cmd_compile(line, raw)
 end
 
 -- Interactive command evaluation
-function eval_cmd(line, raw)
-	local chunk, err = eval_cmd_compile(line, raw)
+function eval_cmd(line, mode)
+	local chunk, err = eval_cmd_compile(line, mode)
 	if not err then
 		return chunk()
 	else
@@ -642,7 +653,7 @@ end
 -- must be public because it is called from eval_cmd()
 -- when map() commands are read from control socket
 function _map_luaobj_call_wrapper(cmd)
-	local func = eval_cmd_compile(cmd, true)
+	local func = eval_cmd_compile(cmd, "RAW")
 	local ret = kluautil.kr_table_pack(xpcall(func, debug.traceback))
 	local ok, serial = pcall(krprint.serialize_lua, ret, 'error')
 	if not ok then
@@ -747,7 +758,7 @@ function map(cmd, format)
 	if (#cmd <= 0) then
 		panic('map() command must be non-empty') end
 	-- syntax check on input command to detect typos early
-	local chunk, err = eval_cmd_compile(cmd, false)
+	local chunk, err = eval_cmd_compile(cmd, "LUA_TABLE")
 	if not chunk then
 		panic('failure when compiling map() command: %s', err)
 	end
@@ -785,7 +796,7 @@ function map(cmd, format)
 		log_info(ffi.C.LOG_GRP_SYSTEM, 'executing map() on %s: command %s', path_name, cmd)
 		local ret
 		if local_exec then
-			ret = eval_cmd(cmd)
+			ret = eval_cmd(cmd, "LUA_TABLE")
 		else
 			ret = map_send_recv(cmd, path)
 			-- skip dead sockets (leftovers from dead instances)
